//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
 
// G4 includes
#include "G4Material.hh"
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4ios.hh"
#include "G4NistManager.hh"

#include "ABlueCubeConstruction.hh"

ABlueCubeConstruction::ABlueCubeConstruction(G4double xc, G4double yc, G4double zc, G4double s, G4String mat):
  X_Center(xc),
  Y_Center(yc),
  Z_Center(zc),
  Side(s),
  MatName(mat),
  solidWorld(0),
  logicWorld(0),
  physiWorld(0),
  WorldMaterial(0),
  CubeMaterial(0)
{

  G4NistManager* man = G4NistManager::Instance();
  WorldMaterial = man->FindOrBuildMaterial("G4_AIR");
  CubeMaterial = man->FindOrBuildMaterial(MatName);

  // to check if the material with the name "MatName" exists in the G4 material database
  if(!CubeMaterial){ 
    const G4String errorMessage("the material of the Blue Cube is undefined -> you asked for the material " + 
				MatName + " but such material does not exist in the GEANT4 database!");

    G4Exception("ABlueCubeConstruction::ABlueCubeConstruction()", 
		"CubeMaterial = man->FindOrBuildMaterial(MatName)", 
		FatalErrorInArgument, 
		errorMessage);  
  }


  // To print some info
  G4cout << G4endl << " ------ INFO ------ Your detector is composed of a Blue Cube of "
	 << MatName << " centered at " << X_Center << " " << Y_Center << " " << Z_Center << " and with Side " << Side << G4endl;
}

ABlueCubeConstruction::~ABlueCubeConstruction()
{
  ; // nothing to be deleted
}


G4VPhysicalVolume* ABlueCubeConstruction::Construct()
{
  // this is the shape of the world. Make sure it is large enough to contain everything
  const G4double HalfWorldLength = sqrt( X_Center*X_Center + Y_Center*Y_Center + Z_Center*Z_Center) + 2*Side;
    
  solidWorld = new G4Box("TheWorld", HalfWorldLength, HalfWorldLength, HalfWorldLength);
  logicWorld = new G4LogicalVolume(solidWorld, WorldMaterial, "TheWorld", 0, 0, 0);
	
  logicWorld->SetVisAttributes(G4VisAttributes::Invisible); // hide the world
	
  //  Must place the World Physical volume unrotated at (0,0,0).
  physiWorld = new G4PVPlacement(0,         // no rotation
				 G4ThreeVector(), // at (0,0,0)
				 logicWorld,      // its logical volume
				 "TheWorld",      // its name
				 0,               // its mother  volume
				 false,           // no boolean operations
				 -1);              // copy number
	
  // Now add a blue cube to the world
  G4Box *asolidBox; 
  G4LogicalVolume *alogicBox; 
  G4VPhysicalVolume *aphysiBox; 
  G4VisAttributes *visatt;
	
  asolidBox = new G4Box("BlueCube",Side/2.,Side/2.,Side/2.);
  alogicBox = new G4LogicalVolume(asolidBox, CubeMaterial, "BlueCube", 0, 0, 0);

  // the cube is blue
  visatt = new G4VisAttributes( G4Colour(0.0, 0.0, 1.0) );
  visatt->SetVisibility(true);
  alogicBox->SetVisAttributes( visatt );
    
  aphysiBox = new G4PVPlacement(0,        // no rotation
				G4ThreeVector(X_Center, Y_Center, Z_Center), // at (X_Center,Y_Center,Z_Center)
				alogicBox,     // its logical volume
				"BlueCube",     // its name
				logicWorld,      // its mother  volume
				false,           // no boolean operations
				0);              // copy number


  // The Physical World is returned there ... this is what G4 Kernel needs to works with your simulations
  return physiWorld;
}



